#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void calc_lambdas(const double is, double elambda[], double elambda1[]);
void calc_thetas(const int z1, const int z2, double elambda[], double elambda1[], double& etheta, double& etheta_prime);

int main() {
   double elambda[17], elambda1[17], etheta, etheta_prime, is;
   int z1, z2;

   while (printf("Enter I, z1, z2: ") && scanf("%lf %i %i", &is, &z1, &z2) == 3) {
      if (abs(z1) <= 4 && abs(z2) <= 4 && is > 0) {
         calc_lambdas(is, elambda, elambda1);
         calc_thetas(z1, z2, elambda, elambda1, etheta, etheta_prime);
         printf("E-theta(I) = %lf, E-theta'(I) = %f\n\n", etheta, etheta_prime);
      }
      else
         printf("Input data out of range\n");
   }
}

void calc_lambdas(const double is, double elambda[], double elambda1[]) {

   // Coefficients c1-c4 are used to approximate the integral function "J";
   //  aphi is the Debye-Huckel constant at 25 C

   const double c1 = 4.581, c2 = 0.7237, c3 = 0.0120, c4 = 0.528;
   const double aphi = 0.392;   /* Value at 25 C */

   // Calculate E-lambda terms for charge combinations of like sign,
   // using method of Pitzer (1975).

   for (int i = 1; i <= 4; i++) {
      for (int j = i; j <= 4; j++) {
         const int ij = i * j;
         const double zprod = (double)ij;
         const double x = 6.0 * zprod * aphi * sqrt(is);                                /* eqn 23 */
         const double jfunc = x / (4.0 + c1 * pow(x, -c2) * exp(-c3 * pow(x, c4)));     /* eqn 47 */

         const double t = c3 * c4 * pow(x, c4);
         const double dj = c1 * pow(x, (-c2 - 1.0)) * (c2 + t) * exp(-c3 * pow(x, c4));
         const double jprime = (jfunc / x) * (1.0 + jfunc * dj);

         elambda[ij] = zprod * jfunc / (4.0 * is);                                      /* eqn 14 */
         elambda1[ij] = (3.0 * zprod * zprod * aphi * jprime / (4.0 * sqrt(is)) - elambda[ij]) / is;
      }
   }
}

void calc_thetas(const int z1, const int z2, double elambda[], double elambda1[],
                 double& etheta, double& etheta_prime) {

   // Calculate E-theta(I) and E-theta'(I) using method of Pitzer (1987)

   if (z1 * z2 < 0) {
      etheta = 0.0;
      etheta_prime = 0.0;
   }
   else {
      const int i = abs(z1);
      const int j = abs(z2);
      const double f1 = (double)i / (2.0 * (double)j);
      const double f2 = (double)j / (2.0 * (double)i);                                 /* eqn A14 */
      etheta = elambda[i * j] - f1 * elambda[j * j] - f2 * elambda[i * i];
      etheta_prime = elambda1[i * j] - f1 * elambda1[j * j] - f2 * elambda1[i * i];
   }
}